from django.utils.translation import gettext as _
from rest_framework import fields, serializers

from . import models


class NANOIDField(fields.Field):

    def to_representation(self, value):
        return str(value)

    def to_internal_value(self, data):
        return data


serializers.ModelSerializer.serializer_field_mapping[models.NANOIDField] = NANOIDField



import nanoid
from django.db import models
from django.utils.translation import gettext as _


class NANOIDField(models.Field):

    empty_strings_allowed = False

    def __init__(self, verbose_name=None, **kwargs):
        self.editable = kwargs.pop('editable', True)
        self.secure_generated = kwargs.pop('secure_generated', True)
        self.alphabetically = kwargs.pop('alphabetically', None)
        self.size = kwargs.pop('size', None)
        self.prefix = kwargs.pop('prefix', None)

        if self.size and not isinstance(self.size, int):
            raise ValueError("Size parameter must be an integer.")

        if self.alphabetically and not isinstance(self.alphabetically, str):
            raise ValueError("Alphabetically parameter must be an string.")

        if self.size and not self.alphabetically:
            raise ValueError("Need to specify a alphabetically parameter")

        if self.alphabetically and not self.size:
            raise ValueError("Need to specify a size parameter")

        kwargs.setdefault('max_length', 21)
        super().__init__(verbose_name, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        del kwargs['max_length']
        kwargs['secure_generated'] = self.secure_generated
        kwargs['alphabetically'] = self.alphabetically
        kwargs['size'] = self.size
        kwargs['prefix'] = self.prefix
        return name, path, args, kwargs

    def get_internal_type(self):
        return 'CharField'

    def generate_nanoid(self):
        if self.secure_generated:
            if self.size and self.alphabetically:
                return nanoid.generate(size=self.size, alphabet=self.alphabetically)
            else:
                return nanoid.generate()
        else:
            if self.size and self.alphabetically:
                return nanoid.non_secure_generate(size=self.size, alphabet=self.alphabetically)
            else:
                return nanoid.non_secure_generate()

    def pre_save(self, model_instance, add):
        value = getattr(model_instance, self.attname)
        if value is None and add:
            value = self.generate_nanoid()
            if self.prefix:
                value = self.prefix + value
            setattr(model_instance, self.attname, value)
        return value

    def get_prep_value(self, value):
        value = super().get_prep_value(value)
        if value is not None and self.alphabetically:
            value = str(value).lower()
        return value



import ast
import re
from setuptools import setup

version_regex = re.compile(r'__version__\s+=\s+(.*)')


def get_version():
    with open('django_nanoid/__init__.py', 'r') as f:
        return str(ast.literal_eval(version_regex.search(f.read()).group(1)))


setup(
    name='django-nanoid',
    version=get_version(),
    author='Yoseph Bernandus',
    author_email='yosephbernandus@gmail.com',
    packages=['django_nanoid'],
    url='https://github.com/yosephbernandus/django-nanoid',
    license='Apache 2.0',
    description='A tiny, secure, URL-friendly, unique string ID generator support in Django',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    package_data={'': ['README.md']},
    install_requires=['Django>=1.9', 'nanoid', 'djangorestframework>=3.0.0'],
    extras_require={
        "dev": ["pytest>=7.0", "twine>=4.0.2"],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)